@@ -940,7 +940,6 @@ gl_hist_init(int size, int beep)
 void
 gl_histadd(const char *buf)
 {
-    static char *prev = 0;
     const char *p = buf;
 
     /* in case we call gl_histadd() before we call getline() */
@@ -952,7 +951,6 @@ gl_histadd(const char *buf)
 	p++;
     if (*p) {
 	hist_buf[hist_last] = hist_save(buf);
-	prev = hist_buf[hist_last];
 	hist_last = hist_last + 1;
 	if(hist_last > HIST_SIZE - 1) {
 	    int i, size = HIST_SIZE + 512;
