@@ -14,40 +14,41 @@
 #  A copy of the GNU General Public License is available at
 #  http://www.r-project.org/Licenses/
 
-.saveRDS <-
-function(object, file = "", ascii = FALSE, version = NULL,
-         compress = TRUE, refhook = NULL)
+saveRDS <-
+    function(object, file = "", ascii = FALSE, version = NULL,
+             compress = TRUE, refhook = NULL)
 {
     if(is.character(file)) {
         if(file == "") stop("'file' must be non-empty string")
         mode <- if(ascii) "w" else "wb"
-        con <- if(compress) gzfile(file, mode) else file(file, mode)
+        con <- if (identical(compress, "bzip2")) bzfile(file, mode)
+            else if (identical(compress, "xz")) xzfile(file, mode)
+            else if(compress) gzfile(file, mode) else file(file, mode)
         on.exit(close(con))
     }
     else if(inherits(file, "connection")) {
+        if (!missing(compress))
+            warning("'compress' is ignored unless 'file' is a file name")
         con <- file
-        if(missing(ascii)) ascii <- summary(con)$text == "text"
     }
     else
         stop("bad 'file' argument")
     invisible(.Internal(serializeToConn(object, con, ascii, version, refhook)))
 }
 
-.readRDS <-
-function(file, refhook = NULL)
+readRDS <- function(file, refhook = NULL)
 {
     if(is.character(file)) {
         con <- gzfile(file, "rb")
         on.exit(close(con))
-    }
-    else if(inherits(file, "connection"))
-        con <- gzcon(file)
-    else
-        stop("bad 'file' argument")
+    } else if(inherits(file, "connection"))
+        con <- file
+    else stop("bad 'file' argument")
     .Internal(unserializeFromConn(con, refhook))
 }
 
-serialize <- function(object, connection, ascii = FALSE, refhook = NULL)
+serialize <-
+    function(object, connection, ascii = FALSE, version = NULL, refhook = NULL)
 {
     if (!is.null(connection)) {
         if (!inherits(connection, "connection"))
@@ -55,9 +56,10 @@ serialize <- function(object, connection, ascii = FALSE, refhook = NULL)
         if (missing(ascii)) ascii <- summary(connection)$text == "text"
     }
     if (!ascii && inherits(connection, "sockconn"))
-        .Call("R_serializeb", object, connection, refhook, PACKAGE="base")
+        .Call("R_serializeb", object, connection, version, refhook,
+              PACKAGE="base")
     else
-        .Call("R_serialize", object, connection, ascii, refhook,
+        .Call("R_serialize", object, connection, ascii, version, refhook,
               PACKAGE="base")
 }
 
