@@ -80,7 +80,11 @@ untar <- function(tarfile, files = NULL, list = FALSE, exdir = ".",
     } else {
         cmd <- paste(TAR, " -", cflag, "xf ", shQuote(tarfile), sep = "")
         if (!missing(exdir)) {
-            dir.create(exdir, showWarnings = FALSE, recursive = TRUE)
+            if (!file_test("-d", exdir)) {
+                if(!dir.create(exdir, showWarnings = TRUE, recursive = TRUE))
+                    stop(gettextf("failed to create directory %s", sQuote(exdir)),
+                         domain = NA)
+            }
             cmd <- if(.Platform$OS.type == "windows")
                 ## some versions of tar.exe need / here
                 paste(cmd, "-C", gsub("\\", "/", exdir, fixed=TRUE))
@@ -116,6 +120,15 @@ untar2 <- function(tarfile, files = NULL, list = FALSE, exdir = ".")
         x
     }
 
+    mydir.create <- function(path, ...) {
+        ## for Windows' sake
+        path <- sub("[\\/]$", "", path)
+        if(file_test("-d", path)) return()
+        if(!dir.create(path, showWarnings = TRUE, recursive = TRUE, ...))
+           stop(gettextf("failed to create directory %s", sQuote(path)),
+                domain = NA)
+    }
+
     warn1 <- character()
 
     ## A tar file is a set of 512 byte records,
@@ -127,7 +140,7 @@ untar2 <- function(tarfile, files = NULL, list = FALSE, exdir = ".")
     } else if(inherits(tarfile, "connection")) con <- tarfile
     else stop("'tarfile' must be a character string or a connection")
     if (!missing(exdir)) {
-        dir.create(exdir, showWarnings = FALSE, recursive = TRUE)
+        mydir.create(exdir)
         od <- setwd(exdir)
         on.exit(setwd(od), add = TRUE)
     }
@@ -152,7 +165,7 @@ untar2 <- function(tarfile, files = NULL, list = FALSE, exdir = ".")
         size <- getOct(block, 124, 12)
         ts <- getOct(block, 136, 12)
         ft <- as.POSIXct(as.numeric(ts), origin="1970-01-01", tz="UTC")
-        csum <- getOct(block, 148, 6)
+        csum <- getOct(block, 148, 8)
         block[149:156] <- charToRaw(" ")
         xx <- as.integer(block)
         checksum <- sum(xx) %% 2^24 # 6 bytes
@@ -172,8 +185,7 @@ untar2 <- function(tarfile, files = NULL, list = FALSE, exdir = ".")
             dothis <- !list
             if(dothis && length(files)) dothis <- name %in% files
             if(dothis) {
-                dir.create(dirname(name), showWarnings = FALSE,
-                           recursive = TRUE)
+                mydir.create(dirname(name))
                 out <- file(name, "wb")
             }
             for(i in seq_len(ceiling(size/512L))) {
@@ -187,7 +199,7 @@ untar2 <- function(tarfile, files = NULL, list = FALSE, exdir = ".")
             }
             if(dothis) {
                 close(out)
-                Sys.chmod(name, mode)
+                Sys.chmod(name, mode, FALSE) # override umask
                 .Call("R_setFileTime", name, ft, PACKAGE = "base")
             }
         } else if(ctype %in% c("1", "2")) { # hard and symbolic links
@@ -198,21 +210,38 @@ untar2 <- function(tarfile, files = NULL, list = FALSE, exdir = ".")
             if(!is.null(llink)) {name2 <- llink; llink <- NULL}
             if(!list) {
                 if(ctype == "1") {
-                    file.copy(name2, name)
-                    warn1 <- c(warn1, "restoring hard link as a file copy")
+                    if (!file.link(name2, name)) { # will give a warning
+                        ## link failed, so try a file copy
+                        if(file.copy(name2, name))
+                             warn1 <- c(warn1, "restoring hard link as a file copy")
+                        else
+                            warning(gettextf("failed to copy %s to %s", sQuote(name2), sQuote(name)), domain = NA)
+                    }
                 } else {
-                    ## this will not work for links to dirs on Windows
                     if(.Platform$OS.type == "windows") {
-                        file.copy(file.path(dirname(name), name2), name)
-                        warn1 <- c(warn1, "restoring symbolic link as a file copy")
-                   } else file.symlink(name2, name)
+                        ## this will not work for links to dirs
+                        from <- file.path(dirname(name), name2)
+                        if (!file.copy(from, name))
+                            warning(gettextf("failed to copy %s to %s", sQuote(from), sQuote(name)), domain = NA)
+                        else
+                            warn1 <- c(warn1, "restoring symbolic link as a file copy")
+                   } else {
+                       if(!file.symlink(name2, name)) { # will give a warning
+                        ## so try a file copy: will not work for links to dirs
+                        from <- file.path(dirname(name), name2)
+                        if (file.copy(from, name))
+                            warn1 <- c(warn1, "restoring symbolic link as a file copy")
+                           else
+                               warning(gettextf("failed to copy %s to %s", sQuote(from), sQuote(name)), domain = NA)
+                       }
+                   }
                 }
             }
         } else if(ctype == "5") {
             contents <- c(contents, name)
             if(!list) {
-                dir.create(name, showWarnings = FALSE, recursive = TRUE)
-                Sys.chmod(name, mode)
+                mydir.create(name)
+                Sys.chmod(name, mode, TRUE) # FIXME: check result
                 ## not much point, since dir will be populated afterwards
                 ## .Call("R_setFileTime", name, ft)
             }
@@ -239,11 +268,11 @@ untar2 <- function(tarfile, files = NULL, list = FALSE, exdir = ".")
 
 tar <- function(tarfile, files = NULL,
                 compression = c("none", "gzip", "bzip2", "xz"),
-                compression_level = 6, tar = Sys.getenv("tar"))
+                compression_level = 6, tar = Sys.getenv("tar"),
+                extra_flags = "")
 {
     if(is.character(tarfile)) {
-        TAR <- tar
-        if(nzchar(TAR) && TAR != "internal") {
+        if(nzchar(tar) && tar != "internal") {
             ## FIXME: could pipe through gzip etc: might be safer for xz
             ## as -J was lzma in GNU tar 1.20:21
             flags <- switch(match.arg(compression),
@@ -251,24 +280,30 @@ tar <- function(tarfile, files = NULL,
                             "gzip" = "-zcf",
                             "bzip2" = "-jcf",
                             "xz" = "-Jcf")
-            cmd <- paste(TAR, flags, shQuote(tarfile),
+
+            if (grepl("darwin", R.version$os)) {
+                ## precaution for Mac OS X to omit resource forks
+                ## we can't tell the running OS version from R.version$os
+                ## but at least it will not be older
+                tar <- paste("COPYFILE_DISABLE=1", tar) # >= 10.5, Leopard
+                if (grepl("darwin8", R.version$os)) # 10.4, Tiger
+                    tar <- paste("COPY_EXTENDED_ATTRIBUTES_DISABLE=1", tar)
+            }
+            cmd <- paste(tar, extra_flags, flags, shQuote(tarfile),
                          paste(shQuote(files), collapse=" "))
             return(invisible(system(cmd)))
         }
         con <- switch(match.arg(compression),
                       "none" =    file(tarfile, "wb"),
-                      "gzip" =  gzfile(tarfile, "wb", compress = compression_level),
-                      "bzip2" = bzfile(tarfile, "wb", compress = compression_level),
-                      "xz" =    xzfile(tarfile, "wb", compress = compression_level))
+                      "gzip" =  gzfile(tarfile, "wb", compression = compression_level),
+                      "bzip2" = bzfile(tarfile, "wb", compression = compression_level),
+                      "xz" =    xzfile(tarfile, "wb", compression = compression_level))
         on.exit(close(con))
     } else if(inherits(tarfile, "connection")) con <- tarfile
     else stop("'tarfile' must be a character string or a connection")
 
     files <- list.files(files, recursive = TRUE, all.files = TRUE,
-                        full.names = TRUE)
-    ## this omits directories: get back the non-empty ones
-    bf <- unique(dirname(files))
-    files <- c(bf[!bf %in% c(".", files)], files)
+                        full.names = TRUE, include.dirs = TRUE)
 
     for (f in unique(files)) {
         info <- file.info(f)
